/*
 * sigmirc.c - send a signal to all running mIRC processes
 * Copyright (c) 2004, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <windows.h>

#define WM_MCOMMAND (WM_USER+200)

#define MAPSZ 1024

int WINAPI WinMain(HINSTANCE hinst, HINSTANCE hprev, LPSTR cmdline, int show) {
  HANDLE hmap, hmutex;
  HWND hwnd = NULL;
  char *ptr;
  int len;

  /* make sure there's at least a signal name on the commandline */
  if (!*cmdline) {
    DWORD written;
    char usage[] = "usage: sigmirc <signal> [data]\r\n";

    WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), usage, lstrlen(usage),
     &written, NULL);

    return 0;
  }

  /* create a file mapping */
  hmap = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
   0, MAPSZ, "mIRC");

  /* check to see the map is newly created, now replaced with mutex below */

  /* map a view of the file mapping onto memory */
  ptr = (char *)MapViewOfFile(hmap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
  if (!ptr) {
    CloseHandle(hmap);
    return 1;
  }

  /* create or open a mutex that will protect the shared memory access */
  if ((hmutex = CreateMutex(NULL, FALSE, "mIRC_SendMessage")) != NULL) {

    /* wait for this mutex to be released - if it is currently locked, wait */
    WaitForSingleObject(hmutex, INFINITE);

    /* store the signal command into memory */
    len = lstrlen(lstrcpy(ptr, "/.signal "));
    lstrcpyn(ptr + len, cmdline, MAPSZ - len);

    /* send the signal to all mIRC windows */
    while ((hwnd = FindWindowEx(NULL, hwnd, "mIRC", NULL)) != NULL)
     SendMessage(hwnd, WM_MCOMMAND, 1, NULL);

    /* get rid of the mutex */
    ReleaseMutex(hmutex);

    CloseHandle(hmutex);
  }

  /* and clean up the file mapping */
  UnmapViewOfFile(ptr);

  CloseHandle(hmap);

  return 0;
}
